#! perl

use Test::More tests => 12;
use Test::Fatal;

use strict;
use warnings;

my @energy;

BEGIN { use_ok( 'MST::Raytrace', 'read_energy_mlcfg' ) }

# every thing is ok
is(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig.cnf',
            layer => 'Ir',
            shell => '1',
            geo   => 'p'
        );
    },
    undef,
    'good shell & geo'
);

# bad file
my $e =
    exception {
        read_energy_mlcfg(
            file  => 'does not exist',
            layer => 'Ir',
        );
    };
ok ( $e->isa( 'Config::Wild::Error::exists' ), 'bad file' );

# bad mirror shell
like(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig.cnf',
            layer => 'Ir',
            shell => '9',
        );
    },
    qr/no multi-layer db/,
    'bad shell'
);

# bad mirror geometry
like(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig.cnf',
            layer => 'Ir',
            geo   => 'q',
        );
    },
    qr/no multi-layer db/,
    'bad shell'
);

# bad post_en_corr_file
like(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig-01.cnf',
            layer => 'Ir'
        );
    },
    qr{error opening t/data/mlcfg/not_exist.rdb},
    'bad post_en_corr_file'
);

# bad reflect_p1_db
like(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig-02.cnf',
            layer => 'Ir'
        );
    },
    qr{error opening t/data/mlcfg/not_exist.rdb},
    'bad reflect_p1_db'
);

# bad layer id
like(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig.cnf',
            layer => 'Fudge'
        );
    },
    qr{can't find layer},
    'bad layer id'
);

# bad layer number
like(
    exception {
        read_energy_mlcfg(
            file  => 't/data/mlcfg/tsconfig.cnf',
            layer => 8
        );
    },
    qr{can't find layer},
    'bad layer number'
);


# all energies, no post_en_corr
@energy = read_energy_mlcfg(
    file  => 't/data/mlcfg/tsconfig-03.cnf',
    layer => 'Ir',
);
ok( $energy[0] == 0.05 && $energy[-1] == 12 && @energy == 1406,
    'all energies, no post_en_corr' );

# all energies, post_en_corr
@energy = read_energy_mlcfg(
    file  => 't/data/mlcfg/tsconfig.cnf',
    layer => 'ol',
);
ok( $energy[0] == 0.01 && $energy[-1] == 11.8295 && @energy == 453,
    'all energies, post_en_corr' );

# post_en_corr, min & max cutoff
@energy = read_energy_mlcfg(
    file  => 't/data/mlcfg/tsconfig.cnf',
    layer => 'Ir',
    emin  => 0.056,
    emax  => 0.058
);
is_deeply(
    \@energy,
    [
        0.056,  0.0562, 0.0564, 0.0566, 0.0568, 0.0570,
        0.0572, 0.0574, 0.0576, 0.0578,
    ],
    'post_en_core, min & max cutoff'
);

